import six

BATCH_SIZE = 100
SORT = "date desc, id asc"


class Cursor(six.Iterator):

    def __init__(self, solr, query):
        self.solr = solr
        self.query = query
        self.mark = "*"
        self.batch = []
        self.position = 0

    def __iter__(self):
        return self

    def __next__(self):
        if self.batch:
            return self.batch.pop(0)
        else:
            self.pull()
            return self.__next__()

    def pull(self):
        results = self.select()
        if results.nextCursorMark == self.mark:
            raise StopIteration
        self.mark = results.nextCursorMark
        self.batch = results.docs

    def select(self):
        return self.solr.search(str(self.query),
                                cursorMark=self.mark,
                                rows=BATCH_SIZE,
                                sort=SORT)



class Query:

    def __and__(self, other):
        return And(self, other)

    def __or__(self, other):
        return Or(self, other)

    def __invert__(self):
        return Not(self)


class Term(Query):

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return quote(self.value)


class Field(Query):

    def __init__(self, field, *values):
        if len(values) == 0:
            raise ValueError('Must have at least one value.')
        self.field = field
        self.values = values

    def __str__(self):
        if len(self.values) == 1:
            return '{}:{}'.format(self.field, quote(self.values[0]))
        else:
            expression = ' OR '.join(quote(value) for value in self.values)
            return '{}:({})'.format(self.field, expression)


class Range(Query):

    def __init__(self, field, upper_bound, lower_bound):
        self.field = field
        self.upper_bound = Term(upper_bound)
        self.lower_bound = Term(lower_bound)

    def __str__(self):
        return '{}:[{} TO {}]'.format(self.field,
                                      self.upper_bound,
                                      self.lower_bound)


class And(Query):

    def __init__(self, left_operand, right_operand):
        self.left_operand = left_operand
        self.right_operand = right_operand

    def __str__(self):
        return '({} AND {})'.format(self.left_operand, self.right_operand)


class Or(Query):

    def __init__(self, left_operand, right_operand):
        self.left_operand = left_operand
        self.right_operand = right_operand

    def __str__(self):
        return '({} OR {})'.format(self.left_operand, self.right_operand)


class Not(Query):

    def __init__(self, operand):
        self.operand

    def __str__(self):
        return '(NOT {})'.format(self.operand)


def quote(value):
    if ' ' in value:
        return '"{}"'.format(value)
    else:
        return value



from .cursor import Cursor


class Search():

    def __init__(self, solr, query="*:*"):
        self.solr = solr
        self.query = query

    def scan(self):
        """Iterate over documents that match the search."""
        return Cursor(self.solr, self.query)



import setuptools


with open("README.md") as f:
    long_description = f.read()

setuptools.setup(
    name="solr-dsl",
    version="0.0.13",
    author="Elliot Penson",
    author_email="elliotpenson@gmail.com",
    description="Python client for Solr",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/it-com-engineering/solr-dsl",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import requests


class Solr:

    def __init__(self, hostport, collection):
        self.hostport = hostport
        self.collection = collection

    def add(self, documents):
        """Document may be a list of doucments or a single document."""
        url = self.make_url()
        params = {'commit': 'true'}
        response = requests.post(url, params=params, json=documents)
        response.raise_for_status()
        return response

    def make_url(self):
        return 'http://{}/solr/{self.collection}/update'.format(self.hostport)
